(ns coderloop.scientist-sql
  (:refer-clojure :only [])
  (:require rlm.ns-rlm rlm.light-base))
(rlm.ns-rlm/ns-clone rlm.light-base)


(use 'clojure.contrib.sql)               ;;' satisfy prettify
(import 'org.gjt.mm.mysql.Driver)
(use 'clojure.contrib.sql)
(import '[org.joda.time LocalDate ])
(undef distinct case compile drop take sort)
(use 'clojureql.core)
(import 'org.joda.time.DateMidnight)
(import 'java.io.File)
(use '[clojure.java.io :only [copy]])
(use '[clojure.contrib.shell-out :only [sh]])
(use '[clojure.string :only [split]])
(use '[coderloop [utils :only [md5]]])

(defn sql-date [date]
  "Convert any Joda-readable date object (including a string) to a java.sql.Date"
    (java.sql.Date. (.. (LocalDate. date) toDateMidnight toInstant getMillis)))

(def *port* 3306)

(def *host* "localhost")

(def *db-host* "localhost")

(def *db-name* "clojure_rlm")

(def *db* {:classname "com.mysql.jdbc.Driver"
	   :subprotocol "mysql"
	   :subname (str "//" *db-host* ":" *port* "/" *db-name*)
	   :user "root"
	   :password "sql1005025"})

(def *db2* {:classname "com.mysql.jdbc.Driver"
	    :subprotocol "mysql"
	    :subname (str "//" *db-host* ":" *port* "/" "test_rlm")
	    :user "root"
	    :password "sql1005025"})


(def databases [:essay_scientists :scientists :ideas :scientist_whereabouts])

(defmacro no-exceptions
  "Sweet relief like I never knew."
  [& forms]
  `(try ~@forms (catch Exception e# (.printStackTrace e#))))


(defn make-essay-scientists []
  (with-connection
    *db2*
    (no-exceptions (drop-table :essay_scientists))
    (create-table
     :essay_scientists
     [:id "integer"]
     [:name "varchar(255)"]
     [:surname "varchar(255)"])
    (insert-rows :essay_scientists
		 [1 "Isaac" "Newton"]
		 [2 "Albert" "Einstein"])))

(defn make-scientists []
  (with-connection
    *db2*
    (no-exceptions (drop-table :scientists))
    (create-table
     :scientists
     [:name "varchar(255)"]
     [:surname "varchar(255)"]
     [:birth_place "varchar(255)"]
     [:death_place "varchar(255)"]
     [:birth_date "date"]
     [:death_date "date"])
    (insert-rows
     :scientists
     ["Albert" "Einstein" "Ulm" "Princeton" "1879-3-14" "1955-4-18"]
     ["Galileo" "Galilei" "Pisa" "Arcetri" "1564-2-15" "1645-1-8"]
     ["Enrico" "Fermi" "Roma" "Chicago" "1901-9-29" "1954-9-28"]
     ["Isaac" "Newton" "Woolsthorpe-by-Colsterworth" "London" "1643-1-4" "1727-3-31"]
     ["Marx" "Planck" "Kiel" "Göttingen" "1858-4-23" "1947-10-4"])))


(defn make-scientist-whereabouts []
  (with-connection *db2*
    (no-exceptions (drop-table :scientist_whereabouts ))
    (create-table
     :scientist_whereabouts
     [:name "varchar(255)"]
     [:surname "varchar(255)"]
     [:country "varchar(255)"]
     [:immigration_date "date"]
     [:emigration_date "date"])
    (insert-rows
     :scientist_whereabouts
     ["Galileo" "Galilei" "France" "1602-2-18" "1604-4-20"]
     ["Albert" "Einstein" "Uk" "1901-10-22" "1905-5-4"]
     ["Marx" "Planck" "Denmark" "1901-1-31" "1945-7-4"]
     ["Albert" "Einstein" "France" "1908-11-7" "1930-3-12"]
     ["Enrico" "Fermi" "Us" "1924-12-19" "1954-9-28"]
     ["Isaac" "Newton" "Uk" "1688-9-8" "1690-7-12"]
     ["Albert" "Einstein" "US" "1935-10-22" "1955-4-18"]
     ["Isaac" "Newton" "Germany" "1690-9-28" "1705-4-6"])))


(defn make-ideas []
  (with-connection *db2*
    (no-exceptions (drop-table :ideas))
    (create-table
     :ideas
     [:name     "varchar(255)"]
     [:surname  "varchar(255)"]
     [:idea     "varchar(255)"])
    (insert-rows
     :ideas
     ["Albert" "Einstein" "Theory of relativity (general)"]
     ["Enrico" "Fermi" "First nuclear reactor"]
     ["Albert" "Einstein" "Photoelectric effect"]
     ["Isaac" "Newton" "Gravity"]
     ["Galileo" "Galilei" "Telescope"]
     ["Albert" "Einstein" "Theory of relativity (special)"]
     ["Isaac" "Newton" "Dark matter"])))

(defn clear-tables []
  (with-connection *db2* (dorun (map drop-table databases))))

(defn fill-tables []
  (make-ideas)
  (make-essay-scientists)
  (make-scientist-whereabouts)
  (make-scientists))

(defn make-empty-tables []
  (with-connection *db2*
    (create-table
     :ideas
     [:name     "varchar(255)"]
     [:surname  "varchar(255)"]
     [:idea     "varchar(255)"])
    (create-table
     :scientist_whereabouts
     [:name "varchar(255)"]
     [:surname "varchar(255)"]
     [:country "varchar(255)"]
     [:immigration_date "date"]
     [:emigration_date "date"])
    (create-table
     :scientists
     [:name "varchar(255)"]
     [:surname "varchar(255)"]
     [:birth_place "varchar(255)"]
     [:death_place "varchar(255)"]
     [:birth_date "date"]
     [:death_date "date"])
    (create-table
     :essay_scientists
     [:id "integer"]
     [:name "varchar(255)"]
     [:surname "varchar(255)"])))

(def fill-tables-command  "mysql -u root -p clojure_rlm < scientists-a.in")

    

(defmethod < [java.sql.Date  java.sql.Date] [a b]
	   ({-1 true 0 false 1 false}
	   (.compareTo a b)))

(defmethod > [java.sql.Date  java.sql.Date]
  ([a b]
     ({-1 true 0 false 1 false}
      (.compareTo a b))))


;; they can have the same date of birth and death!!!
(defn contemps [birth death]
  (with-connection *db*
    @(select
      (table :scientists)
      (where
       (or
	(and (<= :birth_date birth) (< birth :death_date))
	(and  (< :birth_date death) (<= death :death_date))
	(and  (<=  birth :birth_date) (< :birth_date death))
	(and  (<  birth :death_date) (<= :death_date death)))))))

(defn essay-scientists []
  @(table *db* :essay_scientists))

(defn get-info [table-name s]
  @(select 
    (select (table *db* table-name)
	    (where (= :name (:name s))))
    (where(= :surname (:surname s)))))

(defn century* [#^java.sql.Date d]
  (* 100
     (.getCenturyOfEra
      (LocalDate. d))))


(defn century [#^java.sql.Date d]
  (* 100 (clojure.core/unchecked-divide (+ 1900 (.getYear d)) 100)))



(defn get-scientist-data [s]
  (let [data (first (get-info :scientists s))
	locations (get-info :scientist_whereabouts s)
	ideas (map :idea (get-info :ideas s))
	birth (:birth_date data)
	death (:death_date data)
	name (:name data)
	surname (:surname data)
	contemps
	(remove #(and (= (:name %) name) (= (:surname %) surname))
		(map #(select-keys % [:name :surname])
		     (contemps birth death)))
	century (century birth)]
    {:data data :ideas ideas :locations locations :contemps contemps :century century}))

(defn essay-date* [#^java.sql.Date d]
  (let [t (LocalDate. d)]
    (format "%02d/%02d/%04d" (.getDayOfMonth t) (.getMonthOfYear t) (.getYear t))))

(defn essay-date [#^java.sql.Date d]
  (format "%02d/%02d/%04d" (.getDate d) (inc (.getMonth d)) (+ 1900 (.getYear d))))
  
(defn fucked-sort [m]
  (let [surname (:surname m)
	name (:name m)
	[surname n1] (.split surname " ")
	[name n2] (.split name " ")
	n1 (Integer/parseInt n1)
	n2 (Integer/parseInt n2)]
    [surname n1 name n2]))

(defn lexical-sort [m]
  (let [surname (:surname m)
	name (:name m)]
    [surname name]))
  

(defn-memo essay [s]
  (let [info (get-scientist-data s)
	name (:name (:data info))
	surname (:surname (:data info))]
       (str name
	    " "
	    surname
	    " ["
	    (:birth_place (:data info))
	    " "
	    (essay-date (:birth_date (:data info)))
	    ", "
	    (:death_place (:data info))
	    " "
	    (essay-date (:death_date (:data info)))
	    "]"
	    " is one of the most famous scientists of "
	    (:century info)". "
	    "Between all " name "'s ideas we mention: "
	    (apply str (interpose ", " (sort (:ideas info))))
	    ". "
	    name " lived in: "
	    (apply
	     str
	     (interpose ", "
			(map
			 (fn [entry]
			   (str (:country entry)
				" from "
				(essay-date (:immigration_date entry)) " to "
				(essay-date (:emigration_date entry))))
			 (sort-by :immigration_date  (:locations info)))))
	    ". "
	    name
	    " was a contemporary of "
	    (apply
	     str
	     (interpose
	      ", "
	      (map (fn [entry]
		     (str (:name entry)
			  " "
			  (:surname entry)))
		   
		   (sort-by lexical-sort
			    (:contemps info)))))
	     ".\n\n" )))

(defn essays []
  (map essay (essay-scientists)))


(defn main [[username password db-name]]
  (let [db-host *host*
	db-port *port*
	db {:classname "com.mysql.jdbc.Driver"
	    :subprotocol "mysql"
	    :subname (str "//" db-host ":" db-port "/" db-name)
	    :user username
	    :password password}]
    (binding [*db* db]
      (print (apply str (essays))))))

(def desired-md5 "e436d54f87c86bffb5d1b38d5f3e136b")

(if (command-line?)
  (main *command-line-args*))
    
    
	    

